import 'package:flutter/widgets.dart';
import 'package:lux/lux.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

import 'core.dart';

class EventBusProvider<T extends EventStreamableSource<Object?>> extends SingleChildStatelessWidget
    with LuxProviderSingleChildWidget {
  /// {@macro event_bus_provider}
  const EventBusProvider({
    Key? key,
    required Create<T> create,
    this.child,
    this.lazy = true,
  })  : _create = create,
        _value = null,
        super(key: key, child: child);

  const EventBusProvider.value({
    Key? key,
    required T value,
    this.child,
  })  : _value = value,
        _create = null,
        lazy = true,
        super(key: key, child: child);

  final Widget? child;

  final bool lazy;

  final Create<T>? _create;

  final T? _value;

  static T of<T extends EventStreamableSource<Object?>>(
    BuildContext context, {
    bool listen = false,
  }) {
    try {
      return Provider.of<T>(context, listen: listen);
    } on ProviderNotFoundException catch (e) {
      if (e.valueType != T) rethrow;
      throw FlutterError(
        '''
        EventBusProvider.of() called with a context that does not contain a $T.
        No ancestor could be found starting from the context that was passed to EventBusProvider.of<$T>().

        This can happen if the context you used comes from a widget above the EventBusProvider.

        The context used was: $context
        ''',
      );
    }
  }

  @override
  Widget buildWithChild(BuildContext context, Widget? child) {
    assert(
      child != null,
      '$runtimeType used outside of LuxMultiProvider must specify a child',
    );
    final value = _value;
    return value != null
        ? InheritedProvider<T>.value(
            value: value,
            startListening: _startListening,
            lazy: lazy,
            child: child,
          )
        : InheritedProvider<T>(
            create: _create,
            dispose: (_, eventBus) => eventBus.close(),
            startListening: _startListening,
            lazy: lazy,
            child: child,
          );
  }

  static VoidCallback _startListening(
    InheritedContext<Streamable?> e,
    Streamable value,
  ) {
    final subscription = value.stream.listen(
      (dynamic _) => e.markNeedsNotifyDependents(),
    );
    return subscription.cancel;
  }
}
